@@ -5,76 +5,104 @@
 
     Copyright (C) 2022 John Skilleter
 
-    Stuff not implemented, in priority order:
+    Stuff not working:
 
-        / TODO: 1. When changing direcrories, save current location in current directory and return there when we return to the same directory
-        / TODO: 2. Copy (file or directory)
-        / TODO: 2. Move (file or directory)
-        / TODO: 3. Directory delete - with warning if directory not empty) and/or count of total number of files
-        / TODO: 1. If you exit a directory via Backspace then re-enter it hasn't saved the current position
-        / TODO: 1. Implement inotify as background thread as it is far too slow to run everytime we process a keypress
-        / TODO: 2. Use inotify to check for updates and update panes
+        BUG: 1. If both panes showing same directory, non-current pane doesn't update if current one changes
+        BUG: 2. Allows an attempt to move/copy directory into its own subdirectory
+        BUG: 2. If pane showing a subdirectory and parent is deleted in other pane nothing updates
+        BUG: 3. Inefficient redraw (better, but could be better, although curses hides most of the inefficiencies)
+        BUG: 4. Many untrapped exceptions (e.g. deleting stuff)
+        BUG: 6. Slightly odd scrolling at page top/bottom
+
+    Stuff that's been fixed but not fully tested:
+
+        FIXED: 1. If a tagged file is hidden by a filter it shouldn't be processed by move/copy/delete - check this
+
+    DONE:
+        DONE: 1. If you exit a directory via Backspace then re-enter it hasn't saved the current position
+        DONE: 1. Implement inotify as background thread as it is far too slow to run everytime we process a keypress
+        DONE: 1. When changing direcrories, save current location in current directory and return there when we return to the same directory
+        DONE: 2. Copy (file or directory)
+        DONE: 2. Move (file or directory)
+        DONE: 2. Use inotify to check for updates and update panes
+        DONE: 3. Directory delete - with warning if directory not empty) and/or count of total number of files
+
+    CONSIDERED, BUT WON'T DO:
+        NOPE: 3. Look at lfm source code to see how it does things (check licence for compatibility first) - Have looked at feature list and 'borrowed' some
+        NOPE: 8. Multiple panes (fixed at 2 to start with)
+        NOPE: 9. GUI version
+
+    TODO:
         TODO: 1. Config file - keyboard config, override apps used to open particular file types
+        TODO: 2. '>' marker on current file or don't use background colours except for current file
+        TODO: 2. Scrolling help window
         TODO: 2. Wildcard tagging/untagging
-        TODO: 3. Allow copy/move to overwrite files (prompt first)
+        TODO: 2. ^R - redraw
+        TODO: 3. '.' show/hide hidden files
+        TODO: 3. '=' sets other pane to same directory as current one
+        TODO: 3. Allow copy/move to overwrite files (prompt first, with Y/N/A)
+        TODO: 3. Configurable shortcuts
         TODO: 3. Create directory
-        TODO: 3. Use pygments/pymentize for syntax highlighting in file viewer
-        TODO: 3. Look at lfm source code to see how it does things (check licence for compatibility first)
         TODO: 3. Menu for things like sort order
         TODO: 3. Search - name or content
         TODO: 3. Sort by date, size, owner, group, cheese preference, etc.
         TODO: 3. Tagging - invert selection
         TODO: 3. Use inotify and cache file data, or at least use inotify and update when current directory changes
+        TODO: 3. Use pygments/pymentize for syntax highlighting in file viewer
         TODO: 3. When moving between panes move to same line on screen in new pane if possible
-        TODO: 4. Main loop is inefficient - we call filemanager.show_file_list() on every event
         TODO: 4. Bookmarks for directories
+        TODO: 4. F12 = file info = show permissions, times, etc. in pop-up
         TODO: 4. File list - proper formatting, nicer dates/times, support uid, gid, mode, size adjusted for KiB/MiB/GiB and with commas
         TODO: 4. File search/grep
         TODO: 4. Highlight current pane - draw border around it?
+        TODO: 4. Main loop is inefficient - we call filemanager.show_file_list() on every event
         TODO: 4. Menu system with menu bar and pull-down menus
         TODO: 4. Rename
         TODO: 4. Tab to switch panes, left arrow to move up a directory level, right arrow to move into directory under cursor (if it is one)
         TODO: 4. Treat symlinks to directories as directories not files
+        TODO: 4. ^F+string = go to first entry matching 'string'
+        TODO: 4. ^N = go to next ^F/^S match
+        TODO: 4. ^S+strign = go to first entry starting with 'string'
+        TODO: 4. ^U - Swap panes
         TODO: 5. Built-in editor/viewer for text files (can use for help)
         TODO: 5. Create symlink
+        TODO: 5. F11 = directory info = show number of files, size
         TODO: 5. Option to echo cwd on exit so shell can pick up current location as pwd
         TODO: 5. Scroll bar or per-pane status line showing current file index and number of files
+        TODO: 5. Shell window - enter command and run it in current tab
         TODO: 5. Status at bottom of pane, tagged, hiddenm, total files & directories
+        TODO: 5. Verify it works in 16-colour terminal
         TODO: 6. Config file - saves bookmarks, current keyboard config
         TODO: 6. Configuration load/save - global, not directory-specific
         TODO: 6. Control file details visible (user/group/size/permissions/times)
+        TODO: 6. F12 window with edit option to change permissions, owner
+        TODO: 6. Consider splitting TFM out from skilleter-thingy as a separate project; I don't think there's any overlap with the rest of skilleter-thingy
         TODO: 6. File viewer
         TODO: 6. File/directory info
         TODO: 6. More command line parameters & options (directrories, search order, filters)
         TODO: 6. Open/close tab
         TODO: 6. Pickle and unpickle state
+        TODO: 6. Save settings in config file - bookmarks, key configuration, colours, etc.
+        TODO: 6. Search for file, wildcards, show list, select and move to directory
         TODO: 6. Swap panes
         TODO: 6. Synchronised scrolling across multiple panes
+        TODO: 6. Tree view
         TODO: 6. Tree view for navigation
         TODO: 6. Update progress when copying/moving/deleting
+        TODO: 7. Configurable colours
         TODO: 7. Navigation by partial name (press prefix then start typing name and it moves to matching file)
         TODO: 7. Use pads for panes (might be expensive in directories with lots of files)
         TODO: 8. Allow copy/move to merge directories
         TODO: 8. Better help
         TODO: 8. Chmod/chown
-        TODO: 8. Multiple panes (fixed at 2 to start with)
-        TODO: 8. Trying to suspend tfm, run a command in the current shell and resume it doesn't work since I added threads for keyboard and inotify
+        TODO: 8. Directory bookmarks with selection menu
+        TODO: 8. Special environment vars in shell window  - e.g. $:d for current directory $:od for current directory in other pane, $:f - current file $:of current file in other pane
         TODO: 8. Touch file
-        TODO: 9. GUI version
+        TODO: 8. Trying to suspend tfm, run a command in the current shell and resume it doesn't work since I added threads for keyboard and inotify
         TODO: 9. Git commands
         TODO: 9. Mouse support
-    Stuff not working:
+        TODO: 9. multigit integration
 
-        BUG: If both panes showing same directory, non-current pane doesn't update if current one changes
-        BUG: If pane showing a subdirectory and parent is deleted in other pane nothing updates
-        BUG: Inefficient redraw (better, but could be better, although curses hides most of the inefficiencies)
-        BUG: Many untrapped exceptions (e.g. deleting stuff)
-        BUG: Slightly odd scrolling at page top/bottom
-        BUG: Allows an attempt to move/copy directory into its own subdirectory
-
-    Stuff that's been fixed but not fully tested:
-
-        FIXED: If a tagged file is hidden by a filter it shouldn't be processed by move/copy/delete - check this
 """
 ################################################################################
 
@@ -102,6 +130,7 @@ COLOUR_WARNING = 4
 RESERVED_COLOURS = 5
 
 # TODO: Find a better way of sharing colours with the tfm_pane class
+
 PANE_COLOURS = {'normal': 1, 'status': 2, 'background': 3, 'warning': 4, 'reserved_colours': 5}
 
 # Version - used to update old pickle data
