@@ -21,10 +21,15 @@ import re
 # Constants
 
 _ANSI_NORMAL = '\x1b[0m'
+
 _ANSI_BOLD = '\x1b[1m'
+_ANSI_FAINT = '\x1b[2m'
+_ANSI_ITALIC = '\x1b[3m'
 _ANSI_UNDERSCORE = '\x1b[4m'
 _ANSI_BLINK = '\x1b[5m'
 _ANSI_REVERSE = '\x1b[7m'
+_ANSI_STRIKE = '\x1b[9m'
+
 _ANSI_BLACK = '\x1b[30m'
 _ANSI_RED = '\x1b[31m'
 _ANSI_GREEN = '\x1b[32m'
@@ -33,6 +38,16 @@ _ANSI_BLUE = '\x1b[34m'
 _ANSI_MAGENTA = '\x1b[35m'
 _ANSI_CYAN = '\x1b[36m'
 _ANSI_WHITE = '\x1b[37m'
+
+_ANSI_BLACK_B = '\x1b[1;30m'
+_ANSI_RED_B = '\x1b[1;31m'
+_ANSI_GREEN_B = '\x1b[1;32m'
+_ANSI_YELLOW_B = '\x1b[1;33m'
+_ANSI_BLUE_B = '\x1b[1;34m'
+_ANSI_MAGENTA_B = '\x1b[1;35m'
+_ANSI_CYAN_B = '\x1b[1;36m'
+_ANSI_WHITE_B = '\x1b[1;37m'
+
 _ANSI_BBLACK = '\x1b[40m'
 _ANSI_BRED = '\x1b[41m'
 _ANSI_BGREEN = '\x1b[42m'
@@ -42,6 +57,15 @@ _ANSI_BMAGENTA = '\x1b[45m'
 _ANSI_BCYAN = '\x1b[46m'
 _ANSI_BWHITE = '\x1b[47m'
 
+_ANSI_BBLACK_B = '\x1b[1;40m'
+_ANSI_BRED_B = '\x1b[1;41m'
+_ANSI_BGREEN_B = '\x1b[1;42m'
+_ANSI_BYELLOW_B = '\x1b[1;43m'
+_ANSI_BBLUE_B = '\x1b[1;44m'
+_ANSI_BMAGENTA_B = '\x1b[1;45m'
+_ANSI_BCYAN_B = '\x1b[1;46m'
+_ANSI_BWHITE_B = '\x1b[1;47m'
+
 _ANSI_CLEAREOL = '\x1b[K'
 
 # Looking up tables for converting textual colour codes to ANSI codes
@@ -50,9 +74,12 @@ ANSI_REGEXES = \
     (
         (r'\[NORMAL:(.*?)\]', r'%s\1%s' % (_ANSI_NORMAL, _ANSI_NORMAL)),
         (r'\[BOLD:(.*?)\]', r'%s\1%s' % (_ANSI_BOLD, _ANSI_NORMAL)),
+        (r'\[FAINT:(.*?)\]', r'%s\1%s' % (_ANSI_FAINT, _ANSI_NORMAL)),
+        (r'\[ITALIC:(.*?)\]', r'%s\1%s' % (_ANSI_ITALIC, _ANSI_NORMAL)),
         (r'\[UNDERSCORE:(.*?)\]', r'%s\1%s' % (_ANSI_UNDERSCORE, _ANSI_NORMAL)),
         (r'\[BLINK:(.*?)\]', r'%s\1%s' % (_ANSI_BLINK, _ANSI_NORMAL)),
         (r'\[REVERSE:(.*?)\]', r'%s\1%s' % (_ANSI_REVERSE, _ANSI_NORMAL)),
+        (r'\[STRIKE:(.*?)\]', r'%s\1%s' % (_ANSI_STRIKE, _ANSI_NORMAL)),
 
         (r'\[BLACK:(.*?)\]', r'%s\1%s' % (_ANSI_BLACK, _ANSI_NORMAL)),
         (r'\[RED:(.*?)\]', r'%s\1%s' % (_ANSI_RED, _ANSI_NORMAL)),
@@ -63,6 +90,15 @@ ANSI_REGEXES = \
         (r'\[CYAN:(.*?)\]', r'%s\1%s' % (_ANSI_CYAN, _ANSI_NORMAL)),
         (r'\[WHITE:(.*?)\]', r'%s\1%s' % (_ANSI_WHITE, _ANSI_NORMAL)),
 
+        (r'\[BLACK_B:(.*?)\]', r'%s\1%s' % (_ANSI_BLACK_B, _ANSI_NORMAL)),
+        (r'\[RED_B:(.*?)\]', r'%s\1%s' % (_ANSI_RED_B, _ANSI_NORMAL)),
+        (r'\[GREEN_B:(.*?)\]', r'%s\1%s' % (_ANSI_GREEN_B, _ANSI_NORMAL)),
+        (r'\[YELLOW_B:(.*?)\]', r'%s\1%s' % (_ANSI_YELLOW_B, _ANSI_NORMAL)),
+        (r'\[BLUE_B:(.*?)\]', r'%s\1%s' % (_ANSI_BLUE_B, _ANSI_NORMAL)),
+        (r'\[MAGENTA_B:(.*?)\]', r'%s\1%s' % (_ANSI_MAGENTA_B, _ANSI_NORMAL)),
+        (r'\[CYAN_B:(.*?)\]', r'%s\1%s' % (_ANSI_CYAN_B, _ANSI_NORMAL)),
+        (r'\[WHITE_B:(.*?)\]', r'%s\1%s' % (_ANSI_WHITE_B, _ANSI_NORMAL)),
+
         (r'\[BBLACK:(.*?)\]', r'%s\1%s' % (_ANSI_BBLACK, _ANSI_NORMAL)),
         (r'\[BRED:(.*?)\]', r'%s\1%s' % (_ANSI_BRED, _ANSI_NORMAL)),
         (r'\[BGREEN:(.*?)\]', r'%s\1%s' % (_ANSI_BGREEN, _ANSI_NORMAL)),
@@ -70,16 +106,28 @@ ANSI_REGEXES = \
         (r'\[BBLUE:(.*?)\]', r'%s\1%s' % (_ANSI_BBLUE, _ANSI_NORMAL)),
         (r'\[BMAGENTA:(.*?)\]', r'%s\1%s' % (_ANSI_BMAGENTA, _ANSI_NORMAL)),
         (r'\[BCYAN:(.*?)\]', r'%s\1%s' % (_ANSI_BCYAN, _ANSI_NORMAL)),
-        (r'\[BWHITE:(.*?)\]', r'%s\1%s' % (_ANSI_BWHITE, _ANSI_NORMAL))
+        (r'\[BWHITE:(.*?)\]', r'%s\1%s' % (_ANSI_BWHITE, _ANSI_NORMAL)),
+
+        (r'\[BBLACK_B:(.*?)\]', r'%s\1%s' % (_ANSI_BBLACK_B, _ANSI_NORMAL)),
+        (r'\[BRED_B:(.*?)\]', r'%s\1%s' % (_ANSI_BRED_B, _ANSI_NORMAL)),
+        (r'\[BGREEN_B:(.*?)\]', r'%s\1%s' % (_ANSI_BGREEN_B, _ANSI_NORMAL)),
+        (r'\[BYELLOW_B:(.*?)\]', r'%s\1%s' % (_ANSI_BYELLOW_B, _ANSI_NORMAL)),
+        (r'\[BBLUE_B:(.*?)\]', r'%s\1%s' % (_ANSI_BBLUE_B, _ANSI_NORMAL)),
+        (r'\[BMAGENTA_B:(.*?)\]', r'%s\1%s' % (_ANSI_BMAGENTA_B, _ANSI_NORMAL)),
+        (r'\[BCYAN_B:(.*?)\]', r'%s\1%s' % (_ANSI_BCYAN_B, _ANSI_NORMAL)),
+        (r'\[BWHITE_B:(.*?)\]', r'%s\1%s' % (_ANSI_BWHITE_B, _ANSI_NORMAL)),
     )
 
 ANSI_CODES = \
     (
         ('[NORMAL]', _ANSI_NORMAL),
         ('[BOLD]', _ANSI_BOLD),
+        ('[FAINT]', _ANSI_FAINT),
+        ('[ITALIC]', _ANSI_ITALIC),
         ('[UNDERSCORE]', _ANSI_UNDERSCORE),
         ('[BLINK]', _ANSI_BLINK),
         ('[REVERSE]', _ANSI_REVERSE),
+        ('[STRIKE]', _ANSI_STRIKE),
 
         ('[BLACK]', _ANSI_BLACK),
         ('[RED]', _ANSI_RED),
@@ -90,6 +138,15 @@ ANSI_CODES = \
         ('[CYAN]', _ANSI_CYAN),
         ('[WHITE]', _ANSI_WHITE),
 
+        ('[BLACK_B]', _ANSI_BLACK_B),
+        ('[RED_B]', _ANSI_RED_B),
+        ('[GREEN_B]', _ANSI_GREEN_B),
+        ('[YELLOW_B]', _ANSI_YELLOW_B),
+        ('[BLUE_B]', _ANSI_BLUE_B),
+        ('[MAGENTA_B]', _ANSI_MAGENTA_B),
+        ('[CYAN_B]', _ANSI_CYAN_B),
+        ('[WHITE_B]', _ANSI_WHITE_B),
+
         ('[BBLACK]', _ANSI_BBLACK),
         ('[BRED]', _ANSI_BRED),
         ('[BGREEN]', _ANSI_BGREEN),
@@ -98,6 +155,15 @@ ANSI_CODES = \
         ('[BMAGENTA]', _ANSI_BMAGENTA),
         ('[BCYAN]', _ANSI_BCYAN),
         ('[BWHITE]', _ANSI_BWHITE),
+
+        ('[BBLACK_B]', _ANSI_BBLACK_B),
+        ('[BRED_B]', _ANSI_BRED_B),
+        ('[BGREEN_B]', _ANSI_BGREEN_B),
+        ('[BYELLOW_B]', _ANSI_BYELLOW_B),
+        ('[BBLUE_B]', _ANSI_BBLUE_B),
+        ('[BMAGENTA_B]', _ANSI_BMAGENTA_B),
+        ('[BCYAN_B]', _ANSI_BCYAN_B),
+        ('[BWHITE_B]', _ANSI_BWHITE_B),
     )
 
 # Regex to match an ANSI control sequence
@@ -210,11 +276,6 @@ def warning(txt, newline=True, stream=sys.stderr, prefix=False):
 ################################################################################
 
 if __name__ == '__main__':
-    write('Foreground: [RED]red [GREEN]green [BLACK]black [NORMAL]normal')
-    write('Background: [BRED]red [BGREEN]green [BBLACK]black [NORMAL]normal')
-
-    write('Foreground: [BBLUE:blue] [RED:red] normal')
-
     for combo in (0, 1, 2):
         print()
         if combo == 0:
@@ -225,21 +286,43 @@ if __name__ == '__main__':
             print('Combinations')
 
         print()
-        for y in range(0, 32):
-            for x in range(0, 8):
-                colour = x + y * 8
+        for y in range(0, 16):
+            for x in range(0, 16):
+                colour = x + y * 16
 
                 if combo == 0:
-                    write(format('[B%d] %04d ' % (colour, colour)), newline=False)
+                    write(format('[B%d]%4d' % (colour, colour)), newline=False)
                 elif combo == 1:
-                    write(format('[%d] %04d ' % (colour, colour)), newline=False)
+                    write(format('[%d]%4d' % (colour, colour)), newline=False)
                 else:
-                    write(format('[B%d] %04d [%d] %04d ' % (colour, colour, 255 - colour, 255 - colour)), newline=False)
+                    write(format('[B%d]%4d[%d]/%4d ' % (colour, colour, 255 - colour, 255 - colour)), newline=False)
 
             write('[NORMAL]')
 
     print()
 
+    write('Foreground: [RED]red [GREEN]green [BLACK]black [NORMAL]normal')
+    write('Background: [BRED]red [BGREEN]green [BBLACK]black [NORMAL]normal')
+
+    write('Foreground: [BBLUE:blue] [RED:red] normal')
+
+    write('Bright foreground: [RED_B]red [GREEN_B]green [BLACK_B]black [NORMAL]normal')
+    write('Bright background: [BRED_B]red [BGREEN_B]green [BBLACK_B]black [NORMAL]normal')
+
+    write('Foreground: [BBLUE:blue_B] [RED:red_B] normal')
+
+    print()
+
+    write('[NORMAL:Normal text]')
+    write('[FAINT:Faint text]')
+    write('[ITALIC:Italic text]')
+    write('[UNDERSCORE:Underscored text]')
+    write('[BLINK:Blinking text]')
+    write('[REVERSE:Reverse text]')
+    write('[STRIKE:Strikethrough text]')
+
+    print()
+
     error('Error message (nothing should be output after this)', status=0)
 
     write('This message should not appear')
