@@ -37,7 +37,80 @@ class RunError(Exception):
 
 ################################################################################
 
-def capture_output(cmd, input_stream, output_streams, ansi_clean):
+# TODO: This is the _process() and run() replacement once additional parameters have been implemented
+# as those functions are probably over-specified, so need to work out what functionality is ACTUALLY being used!
+
+def command(cmd, show_stdout=False, show_stderr=False):
+    """
+    Run an external command and optionally stream its stdout/stderr to the console
+    while capturing them.
+
+    Args:
+        cmd: Command to run (string or argv list). If a string, it will be split with shlex.split().
+        show_stdout: If True, echo stdout lines to the console as they arrive.
+        show_stderr: If True, echo stderr lines to the console as they arrive.
+
+    Returns:
+        (returncode, stdout_lines, stderr_lines)
+    """
+
+    def _pump(stream, sink, echo):
+        """Thread to capture and optionally echo output from subprocess.Popen"""
+
+        # Read line-by-line until EOF
+
+        for line in iter(stream.readline, ""):
+            # Strip trailing newline when storing; keep original when echoing
+            sink.append(line.rstrip("\n"))
+
+            if echo:
+                print(tidy.convert_ansi(line), end="", flush=True)
+
+        stream.close()
+
+    # Normalize command to be a string
+
+    if isinstance(cmd, str):
+        cmd = shlex.split(cmd)
+
+    # Storage for stdout/stderr
+
+    stdout_lines = []
+    stderr_lines = []
+
+    # Start process with separate pipes; line-buffering for timely reads
+
+    proc = subprocess.Popen(
+        cmd,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        text=True,                 # decode to str
+        bufsize=1,                 # line-buffered
+        universal_newlines=True,   # compatibility alias
+        errors="replace"           # avoid crashes on decoding issues
+    )
+
+    # Threads to read both streams concurrently (prevents deadlocks)
+
+    t_out = threading.Thread(target=_pump, args=(proc.stdout, stdout_lines, show_stdout))
+    t_err = threading.Thread(target=_pump, args=(proc.stderr, stderr_lines, show_stderr))
+
+    t_out.start()
+    t_err.start()
+
+    # Wait for process to complete and threads to drain
+
+    returncode = proc.wait()
+    t_out.join()
+    t_err.join()
+
+    # Return the status, stdout and stderr
+
+    return returncode, stdout_lines, stderr_lines
+
+################################################################################
+
+def capture_output(cmd, input_stream, output_streams):
     """ Capture data from a stream (input_stream), optionally
         outputting it (if output_streams is not None and optionally
         saving it into a variable (data, if not None), terminating
@@ -58,8 +131,6 @@ def capture_output(cmd, input_stream, output_streams, ansi_clean):
                     else:
                         if stream in (sys.stdout, sys.stderr):
                             stream.write(tidy.convert_ansi(output))
-                        elif ansi_clean:
-                            stream.write(tidy.remove_ansi(output))
                         else:
                             stream.write(output)
 
@@ -68,77 +139,23 @@ def capture_output(cmd, input_stream, output_streams, ansi_clean):
 
 ################################################################################
 
-def capture_continuous(cmd, input_stream, output_streams, ansi_clean):
-    """ Capture data from a stream (input_stream), optionally
-        outputting it (if output_streams is not None and optionally
-        saving it into a variable (data, if not None), terminating
-        when the specified command (cmd, which is presumed to be the process
-        outputting to the input stream) exits.
-        TODO: Use of convert_ansi should be controlled via a parameter (off/light/dark)
-        TODO: ansi_clean not implemented
-    """
-
-    output_buffer = []
-
-    while True:
-        output = input_stream.read(1)
-
-        if output:
-            if output_streams:
-                for stream in output_streams:
-                    if isinstance(stream, list):
-                        if output == '\n':
-                            stream.append(''.join(output_buffer))
-                            output_buffer = []
-                        else:
-                            output_buffer.append(output)
-                    else:
-                        stream.write(output)
-                        stream.flush()
-
-        elif cmd.poll() is not None:
-            if output_buffer:
-                stream.append(''.join(output_buffer))
-
-            return
-
-################################################################################
-
-def process(command,
-            stdout=None, stderr=None,
-            return_stdout=False, return_stderr=False,
-            shell=False,
-            output=None,
-            ansi_clean=False,
-            exception=True,
-            continuous=False):
+def _process(command,
+             stdout=None, stderr=None,
+             output=None):
     """ Run an external command.
 
         stdout and stderr indicate whether stdout/err are output and/or sent to a file and/or stored in a variable.
         They can be boolean (True: output to sys.stdout/err, False: Do nothing), a file handle or a variable, or an
         array of any number of these (except booleans).
 
-        return_stdout and return_stderr indicate whether stdout/err should be returned from the function (setting
-        these to False saves memory if the output is not required).
-
-        If shell is True the command will be run in a shell and wildcard arguments expanded
-
-        If exception is True an exception will be raised if the command returns a non-zero status
-
         If output is True then stdout and stderr are both output as if stdout=True and stderr=True (in addition to
         any other values passed in those parameters)
 
-        If ansi_clean is True then ANSI control sequences are removed from any streams in stdout and stderr but
-        not from the console output.
-
-        If continuous is True then output is processed character-by-character (normally for use when output=True)
-        TODO: Currently this causes the ansi_clean option to be ignored
-
         The return value is a tuple consisting of the status code, captured stdout (if any) and captured
         stderr (if any).
 
-        Will raise OSError if the command could not be run and RunError if exception is True and the
-        command returned a non-zero status code. """
+        Will raise OSError if the command could not be run and RunError if the command returned a non-zero status code.
+    """
 
     # If stdout/stderr are booleans then output to stdout/stderr if True, else discard output
 
@@ -165,26 +182,16 @@ def process(command,
         if sys.stderr not in stderr:
             stderr.append(sys.stderr)
 
-    # Capture stdout/stderr to arrays unless asked not to
+    # Capture stdout/stderr to arrays
 
     stdout_data = []
     stderr_data = []
 
-    if return_stdout:
-        stdout.append(stdout_data)
+    stdout.append(stdout_data)
+    stderr.append(stderr_data)
 
-    if return_stderr:
-        stderr.append(stderr_data)
-
-    # If running via the shell then the command should be a string, otherwise
-    # it should be an array
-
-    if shell:
-        if not isinstance(command, str):
-            command = ' '.join(command)
-    else:
-        if isinstance(command, str):
-            command = shlex.split(command, comments=True)
+    if isinstance(command, str):
+        command = shlex.split(command, comments=True)
 
     # Use a pipe for stdout/stderr if are are capturing it
     # and send it to /dev/null if we don't care about it at all.
@@ -206,19 +213,18 @@ def process(command,
                            stderr=stderr_stream,
                            text=True,
                            errors='ignore',
-                           encoding='ascii',
-                           shell=shell)
+                           encoding='ascii')
 
     # Create threads to capture stderr and/or stdout if necessary
 
     if stdout_stream == subprocess.PIPE:
-        stdout_thread = threading.Thread(target=capture_continuous if continuous else capture_output, args=(cmd, cmd.stdout, stdout, ansi_clean), daemon=True)
+        stdout_thread = threading.Thread(target=capture_output, args=(cmd, cmd.stdout, stdout), daemon=True)
         stdout_thread.start()
     else:
         stdout_thread = None
 
     if stderr_stream == subprocess.PIPE:
-        stderr_thread = threading.Thread(target=capture_continuous if continuous else capture_output, args=(cmd, cmd.stderr, stderr, ansi_clean), daemon=True)
+        stderr_thread = threading.Thread(target=capture_output, args=(cmd, cmd.stderr, stderr), daemon=True)
         stderr_thread.start()
     else:
         stderr_thread = None
@@ -233,13 +239,10 @@ def process(command,
 
     cmd.wait()
 
-    # If the command failed, raise an exception (if required)
+    # If the command failed, raise an exception
 
-    if exception and cmd.returncode:
-        if return_stderr:
-            raise RunError('\n'.join(stderr_data))
-        else:
-            raise RunError('Error %d running "%s"' % (cmd.returncode, (command if isinstance(command, str) else ' '.join(command))))
+    if cmd.returncode:
+        raise RunError('\n'.join(stderr_data) if stderr_data else 'Error %d running "%s"' % (cmd.returncode, ' '.join(command)))
 
     # Return status, stdout, stderr (the latter 2 may be empty if we did not capture data).
 
@@ -248,107 +251,34 @@ def process(command,
 ################################################################################
 
 def run(command,
-        stdout=None, stderr=None,
-        shell=False,
-        output=None,
-        ansi_clean=False,
-        exception=True,
-        continuous=False):
-    """ Simple interface to the process() function
+        stdout=None,
+        stderr=None,
+        output=None):
+    """ Simple interface to the _process() function
         Has the same parameters, with the same defaults.
         The return value is either the data output to stdout, if any
         or the data output to stderr otherwise.
         The status code is not returned, but the function will raise an exception
-        by default if it is non-zero """
+        if it is non-zero """
 
-    result = process(command=command,
-                     stdout=stdout, stderr=stderr,
-                     return_stdout=True, return_stderr=True,
-                     shell=shell,
-                     output=output,
-                     ansi_clean=ansi_clean,
-                     exception=exception,
-                     continuous=continuous)
+    result = _process(command=command,
+                      stdout=stdout,
+                      stderr=stderr,
+                      output=output)
 
     return result['stdout'] if result['stdout'] else result['stderr']
 
 ################################################################################
 
-def status(command, shell=False, output=False):
-    """ Alternative simple interface to the process() function
-        Just takes a command and the shell flag.
-        Runs the command without capturing the output
-        Optionally outputting both stdout and stderr
-        and returns the status code.
-        Will only raise an exception if the command could not be run. """
-
-    return process(command,
-                   stdout=output,
-                   stderr=output,
-                   shell=shell,
-                   exception=False)['status']
-
-################################################################################
-# Legacy compatibility layer for process.py API
-
-def run_process(command, foreground=False, shell=False):
-    """
-    Legacy compatibility function for process.py API.
-
-    Args:
-        command: Command to run (string or list)
-        foreground: If True, run in foreground with output to console
-        shell: Whether to use shell for execution
-
-    Returns:
-        List of output lines (empty if foreground=True)
-
-    Raises:
-        RunError: If command fails
-    """
-    if foreground:
-        # For foreground mode, output directly to console
-        try:
-            status_result = status(command, shell=shell, output=True)
-            if status_result != 0:
-                raise RunError(f"Command failed with return code {status_result}")
-            return []  # process.py returns empty list for foreground mode
-        except Exception as e:
-            if isinstance(e, RunError):
-                raise
-            raise RunError(f"Command failed: {str(e)}")
-    else:
-        # For background mode, capture and return output
-        try:
-            result = run(command, shell=shell, exception=True)
-            if isinstance(result, list):
-                return [line.rstrip() for line in result if line.strip()]
-            elif isinstance(result, str):
-                return [line.rstrip() for line in result.splitlines() if line.strip()]
-            return []
-        except Exception as e:
-            if isinstance(e, RunError):
-                raise
-            raise RunError(f"Command failed: {str(e)}")
-
-################################################################################
-
 if __name__ == '__main__':
     def test_run(cmd,
-                 stdout=None, stderr=None,
-                 return_stdout=True, return_stderr=True,
-                 shell=False,
-                 exception=True):
+                 stdout=None, stderr=None):
         """ Test wrapper for the process() function. """
 
         print('-' * 80)
         print('Running: %s' % (cmd if isinstance(cmd, str) else ' '.join(cmd)))
 
-        result = process(cmd,
-                         stdout=stdout, stderr=stderr,
-                         return_stdout=return_stdout, return_stderr=return_stderr,
-                         shell=shell,
-                         exception=exception)
+        result = _process(cmd, stdout=stdout, stderr=stderr)
 
         print('Status: %d' % result['status'])
 
@@ -358,19 +288,19 @@ if __name__ == '__main__':
         test_run('echo nothing')
 
         test_run(['ls', '-l', 'run_jed'])
-        test_run(['ls -l run_*'], stdout=True, shell=True)
-        test_run('false', exception=False)
-        test_run('true', stdout=sys.stdout, exception=False)
-        test_run(['git', 'status'], stdout=sys.stdout, stderr=sys.stderr, exception=False)
+        test_run(['ls -l run_*'], stdout=True)
+        test_run('false')
+        test_run('true', stdout=sys.stdout)
+        test_run(['git', 'status'], stdout=sys.stdout, stderr=sys.stderr)
 
-        test_run(['make'], stderr=sys.stderr, exception=False)
-        test_run(['make'], stdout=sys.stdout, stderr=[sys.stderr], exception=False)
-        test_run(['make'], stdout=True, exception=False)
-        test_run(['make'], stdout=sys.stdout, exception=False)
-        test_run(['make'], exception=False)
+        test_run(['make'], stderr=sys.stderr)
+        test_run(['make'], stdout=sys.stdout, stderr=[sys.stderr])
+        test_run(['make'], stdout=True)
+        test_run(['make'], stdout=sys.stdout)
+        test_run(['make'])
 
         output = []
-        test_run('ls -l x*; sleep 1; echo "Bye!"', stderr=[sys.stderr, output], stdout=sys.stdout, shell=True, return_stdout=False)
+        test_run('ls -l x*; sleep 1; echo "Bye!"', stderr=[sys.stderr, output], stdout=sys.stdout)
         print('Output=%s' % output)
 
     test()
